]> sigrok.org Git - libsigrok.git/blobdiff - device.c
Improve sr_filter_probes error handling and docs.
[libsigrok.git] / device.c
index dd3a4000bcc6b9f9405ba673aca5035eedb8acfe..6921ceb0fad60243a8eae13e06adfc2434af4167 100644 (file)
--- a/device.c
+++ b/device.c
 #include <stdio.h>
 #include <glib.h>
 #include <sigrok.h>
+#include <sigrok-internal.h>
 
 extern struct sr_global *global;
 
 GSList *devices = NULL;
 
+
 void sr_device_scan(void)
 {
        GSList *plugins, *l;
        struct sr_device_plugin *plugin;
 
-       plugins = list_hwplugins();
+       plugins = sr_list_hwplugins();
 
        /*
         * Initialize all plugins first. Since the init() call may involve
@@ -39,35 +41,9 @@ void sr_device_scan(void)
         */
        for (l = plugins; l; l = l->next) {
                plugin = l->data;
-               sr_device_plugin_init(plugin);
-       }
-
-}
-
-int sr_device_plugin_init(struct sr_device_plugin *plugin)
-{
-       int num_devices, num_probes, i;
-
-       g_message("initializing %s plugin", plugin->name);
-       num_devices = plugin->init(NULL);
-       for (i = 0; i < num_devices; i++) {
-               num_probes = (int)plugin->get_device_info(i, SR_DI_NUM_PROBES);
-               sr_device_new(plugin, i, num_probes);
+               sr_init_hwplugins(plugin);
        }
 
-       return num_devices;
-}
-
-void sr_device_close_all(void)
-{
-       struct sr_device *device;
-
-       while (devices) {
-               device = devices->data;
-               if (device->plugin && device->plugin->close)
-                       device->plugin->close(device->plugin_index);
-               sr_device_destroy(device);
-       }
 }
 
 GSList *sr_device_list(void)
@@ -85,7 +61,11 @@ struct sr_device *sr_device_new(struct sr_device_plugin *plugin, int plugin_inde
        struct sr_device *device;
        int i;
 
-       device = g_malloc0(sizeof(struct sr_device));
+       if (!(device = g_try_malloc0(sizeof(struct sr_device)))) {
+               sr_err("dev: %s: device malloc failed", __func__);
+               return NULL;
+       }
+
        device->plugin = plugin;
        device->plugin_index = plugin_index;
        devices = g_slist_append(devices, device);
@@ -100,8 +80,6 @@ void sr_device_clear(struct sr_device *device)
 {
        unsigned int pnum;
 
-       /* TODO: Plugin-specific clear call? */
-
        if (!device->probes)
                return;
 
@@ -109,29 +87,11 @@ void sr_device_clear(struct sr_device *device)
                sr_device_probe_clear(device, pnum);
 }
 
-void sr_device_destroy(struct sr_device *device)
-{
-       unsigned int pnum;
-
-       /*
-        * TODO: Plugin-specific destroy call, need to decrease refcount
-        * in plugin.
-        */
-
-       devices = g_slist_remove(devices, device);
-       if (device->probes) {
-               for (pnum = 1; pnum <= g_slist_length(device->probes); pnum++)
-                       sr_device_probe_clear(device, pnum);
-               g_slist_free(device->probes);
-       }
-       g_free(device);
-}
-
 void sr_device_probe_clear(struct sr_device *device, int probenum)
 {
        struct sr_probe *p;
 
-       p = probe_find(device, probenum);
+       p = sr_device_probe_find(device, probenum);
        if (!p)
                return;
 
@@ -146,14 +106,20 @@ void sr_device_probe_clear(struct sr_device *device, int probenum)
        }
 }
 
-void sr_device_probe_add(struct sr_device *device, char *name)
+void sr_device_probe_add(struct sr_device *device, const char *name)
 {
        struct sr_probe *p;
        char probename[16];
        int probenum;
 
        probenum = g_slist_length(device->probes) + 1;
-       p = g_malloc0(sizeof(struct sr_probe));
+
+       if (!(p = g_try_malloc0(sizeof(struct sr_probe)))) {
+               sr_err("dev: %s: p malloc failed", __func__);
+               // return SR_ERR_MALLOC;
+               return; /* FIXME: should return int. */
+       }
+
        p->index = probenum;
        p->enabled = TRUE;
        if (name) {
@@ -166,7 +132,7 @@ void sr_device_probe_add(struct sr_device *device, char *name)
        device->probes = g_slist_append(device->probes, p);
 }
 
-struct sr_probe *probe_find(struct sr_device *device, int probenum)
+struct sr_probe *sr_device_probe_find(struct sr_device *device, int probenum)
 {
        GSList *l;
        struct sr_probe *p, *found_probe;
@@ -183,12 +149,13 @@ struct sr_probe *probe_find(struct sr_device *device, int probenum)
        return found_probe;
 }
 
-/* TODO: return SIGROK_ERR if probenum not found */
-void sr_device_probe_name(struct sr_device *device, int probenum, char *name)
+/* TODO: return SR_ERR if probenum not found */
+void sr_device_probe_name(struct sr_device *device, int probenum,
+                         const char *name)
 {
        struct sr_probe *p;
 
-       p = probe_find(device, probenum);
+       p = sr_device_probe_find(device, probenum);
        if (!p)
                return;
 
@@ -197,7 +164,7 @@ void sr_device_probe_name(struct sr_device *device, int probenum, char *name)
        p->name = g_strdup(name);
 }
 
/* TODO: return SIGROK_ERR if probenum not found */
 void sr_device_trigger_clear(struct sr_device *device)
 {
        struct sr_probe *p;
@@ -207,7 +174,7 @@ void sr_device_trigger_clear(struct sr_device *device)
                return;
 
        for (pnum = 1; pnum <= g_slist_length(device->probes); pnum++) {
-               p = probe_find(device, pnum);
+               p = sr_device_probe_find(device, pnum);
                if (p && p->trigger) {
                        g_free(p->trigger);
                        p->trigger = NULL;
@@ -215,12 +182,13 @@ void sr_device_trigger_clear(struct sr_device *device)
        }
 }
 
-/* TODO: return SIGROK_ERR if probenum not found */
-void sr_device_trigger_set(struct sr_device *device, int probenum, char *trigger)
+/* TODO: return SR_ERR if probenum not found */
+void sr_device_trigger_set(struct sr_device *device, int probenum,
+                          const char *trigger)
 {
        struct sr_probe *p;
 
-       p = probe_find(device, probenum);
+       p = sr_device_probe_find(device, probenum);
        if (!p)
                return;
 
@@ -235,6 +203,9 @@ gboolean sr_device_has_hwcap(struct sr_device *device, int hwcap)
 {
        int *capabilities, i;
 
+       if (!device || !device->plugin)
+               return FALSE;
+
        if ((capabilities = device->plugin->get_capabilities()))
                for (i = 0; capabilities[i]; i++)
                        if (capabilities[i] == hwcap)
@@ -197,7 +164,7 @@ void sr_device_probe_name(struct sr_device *device, int probenum, char *name)
 }
 
-/* TODO: return SIGROK_ERR if probenum not found */
+/* TODO: return SR_ERR if probenum not found */
 void sr_device_trigger_clear(struct sr_device *device)
 {